"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var domain = require("domain");
var vm = require("vm");
var main_module_1 = require("./main-module");
var RecimeError = /** @class */ (function (_super) {
    __extends(RecimeError, _super);
    function RecimeError(message) {
        return _super.call(this, message) || this;
    }
    return RecimeError;
}(Error));
;
var Runnable = /** @class */ (function () {
    function Runnable(bot, options) {
        this.options = options;
        this.bot = bot;
    }
    Runnable.prototype.run = function (data) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var options = _this.options;
            try {
                var mainModule_1 = new main_module_1.MainModule(options);
                fs.readFile(mainModule_1.path(), 'utf-8', function (err, code) {
                    if (err) {
                        return reject(err);
                    }
                    // uid
                    options.env['UID'] = options.uid;
                    options.env['HOME_DIR'] = options.homedir;
                    // pass in platform for external modules to take advantage of.
                    options.env['PLATFORM'] = options.platform || "web";
                    // aws region
                    options.env['REGION'] = _this.bot.region;
                    var p = {
                        env: _this.bot.config
                    };
                    p.env["UID"] = _this.bot.id;
                    p.env["REGION"] = _this.bot.region;
                    var sandbox = {
                        console: console,
                        exports: exports,
                        process: p,
                        require: function (module) {
                            return require(mainModule_1.normalizedDepsPath(module));
                        },
                        url: function (p) {
                            if (process.env.AWS_LAMBDA_FUNCTION_NAME) {
                                var asset = p.replace(/(\/)?assets\//ig, "");
                                return "https://assets.recime.io/" + path.normalize(options.uid + "/" + asset);
                            }
                            return "https://" + options.host + "/" + path.normalize(options.uid + "/v1/" + p);
                        },
                        __dirname: mainModule_1.dir
                    };
                    var script = new vm.Script(code);
                    var context = vm.createContext(sandbox);
                    var Module = script.runInContext(context);
                    if (typeof Module === 'undefined') {
                        reject(new RecimeError("Could not resolve the main module."));
                        return;
                    }
                    var local = domain.create();
                    local.on("error", function (err) {
                        reject(new RecimeError(err.message));
                    });
                    local.run(function () {
                        // export default
                        var Main = Module.Bot || Module;
                        var isInstanceObj = !!Main.prototype && !!Main.prototype.constructor.name;
                        if (isInstanceObj) {
                            var main = new Main(data);
                            if (Main.constructor.length === 0)
                                return reject(new RecimeError("Invalid Bot class definition."));
                            switch (main.constructor.name) {
                                case "Bot": {
                                    var ret = main.execute(resolve);
                                    if (ret && ret.then && main.execute.length === 0) {
                                        ret.then(resolve, function (err) {
                                            reject(new RecimeError(err.message));
                                        });
                                    }
                                    break;
                                }
                                default: {
                                    reject(new RecimeError("Invalid class name " + main.constructor.name));
                                }
                            }
                        }
                        else {
                            return Module(data, resolve);
                        }
                    });
                });
            }
            catch (err) {
                reject(err);
            }
        });
    };
    return Runnable;
}());
exports.Runnable = Runnable;
