"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var crypto = require("crypto");
var request = require("request");
var channel_1 = require("./channel");
var logger_1 = require("./logger");
var recime_keyvalue_store_1 = require("recime-keyvalue-store");
var WeChat = /** @class */ (function (_super) {
    __extends(WeChat, _super);
    function WeChat(bot) {
        return _super.call(this, { bot: bot, log: new logger_1.WeChatLogger(bot) }) || this;
    }
    WeChat.prototype.execute = function (body, parameters, handler) {
        var _this = this;
        return new Promise(function (resolve) {
            _this.context.log.incoming(body);
            handler(_this.context.bot, body, 'wechat', function (message) {
                var messages = [];
                if (!Array.isArray(message)) {
                    if (message) {
                        messages.push(message);
                    }
                }
                else {
                    messages = message;
                }
                ;
                _this.sendMessage(0, body.FromUserName, messages).then(function () {
                    resolve();
                });
            }, function (err) {
                resolve(err.message);
            });
        });
    };
    WeChat.prototype.sendMessage = function (index, toUser, messages) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var message = messages[index];
            var reqData = {
                touser: toUser,
                msgtype: message.type,
            };
            switch (message.type) {
                case "text": {
                    reqData.text = {
                        content: message.content
                    };
                    break;
                }
                case "audio":
                case "image":
                case "video": {
                    message.type = message.type === "audio" ? "voice" : message.type;
                    return _this.uploadMedia(message.url, message.type).then(function (data) {
                        reqData.msgtype = data.type;
                        if (data.media_id) {
                            reqData[message.type] = {
                                media_id: data.media_id
                            };
                        }
                        _this.postMessage(reqData).then(function () {
                            if (index + 1 < messages.length)
                                _this.sendMessage(++index, toUser, messages).then(resolve);
                            else
                                resolve();
                        });
                    });
                }
                case "news": {
                    reqData.news = message.news;
                    break;
                }
                default: {
                    reqData.msgtype = "text";
                    reqData.text = {
                        content: message.text || "Message type not supported."
                    };
                }
            }
            ;
            _this.postMessage(reqData).then(function () {
                if (index + 1 < messages.length)
                    _this.sendMessage(++index, toUser, messages).then(resolve);
                else
                    resolve();
            });
        });
    };
    WeChat.prototype.postMessage = function (data) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.accessToken().then(function (accessToken) {
                var reqData = {
                    url: "https://api.weixin.qq.com/cgi-bin/message/custom/send?access_token=" + accessToken,
                    json: true,
                    method: "POST",
                    body: data
                };
                request(reqData, function (err, response, body) {
                    if (err) {
                        console.error(err);
                        return resolve();
                    }
                    if (response.statusCode < 200 && response.statusCode > 299) {
                        console.error(body);
                        return resolve();
                    }
                    _this.context.log.outgoing(reqData, body);
                    resolve();
                });
            })
                .catch(function (err) {
                console.error(err);
                resolve();
            });
        });
    };
    WeChat.prototype.accessToken = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var key = _this.context.bot.config.RECIME_WECHAT_APP_ID + "|" + _this.context.bot.config.RECIME_WECHAT_APP_SECRET;
            recime_keyvalue_store_1.default.get(key).then(function (value) {
                if (value && value.expires > new Date().getTime())
                    resolve(value.token);
                else {
                    request.get("https://proxy.recime.io/wechat/token?grant_type=client_credential&appid=" + (process.env.RECIME_WECHAT_APP_ID || _this.context.bot.config.RECIME_WECHAT_APP_ID) + "&secret=" + (process.env.RECIME_WECHAT_APP_SECRET || _this.context.bot.config.RECIME_WECHAT_APP_SECRET), {
                        "Accept": "application/json"
                    }, function (err, response, body) {
                        var result = JSON.parse(body);
                        if (response.statusCode < 200 && response.statusCode > 299)
                            return reject(new Error("[Error] Invalid access token"));
                        if (result.errcode)
                            return reject(new Error(result.errmsg));
                        var accessToken = JSON.parse(response.body).access_token;
                        recime_keyvalue_store_1.default.set(key, {
                            expires: Date.now() + 7200,
                            token: accessToken
                        }).then(function () { return resolve(accessToken); });
                        resolve(accessToken);
                    });
                }
            });
        });
    };
    WeChat.prototype.uploadMedia = function (mediaUrl, type) {
        var _this = this;
        var parsed = require("url").parse(mediaUrl).path;
        return new Promise(function (resolve) {
            _this.accessToken().then(function (accessToken) {
                var readStream = request(mediaUrl);
                request({
                    url: "https://api.weixin.qq.com/cgi-bin/media/upload?access_token=" + accessToken + "&type=" + type,
                    json: true,
                    method: "POST",
                    formData: {
                        media: readStream,
                        filename: path.basename(parsed)
                    }
                }, function (err, response, body) {
                    if (response.statusCode < 200 && response.statusCode > 299) {
                        return resolve({
                            msgtype: "text",
                            content: "[Error] Invalid media url."
                        });
                    }
                    if (body.errcode) {
                        return resolve({
                            msgtype: "text",
                            content: body.errmsg
                        });
                    }
                    return resolve(body);
                });
            }, function (err) {
                return resolve({
                    msgtype: "text",
                    content: err.message
                });
            });
        });
    };
    WeChat.prototype.verify = function (query) {
        var accessToken = process.env.RECIME_WECHAT_ACCESS_TOKEN || this.context.bot.config.RECIME_WECHAT_ACCESS_TOKEN;
        var token = [query['timestamp'], query['nonce'], accessToken].sort().join("");
        var hash = crypto.createHash('sha1').update(token).digest('hex');
        return hash === query['signature'];
    };
    return WeChat;
}(channel_1.Channel));
exports.WeChat = WeChat;
